#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.08 22:00:00                  #
# ================================================== #

from PySide6.QtWidgets import QTreeView, QGridLayout, QAbstractItemView, QScrollArea

from ..widget.dialog import DebugDialog


class Debug:
    def __init__(self, window=None):
        """
        Debug setup

        :param window: main UI window object
        """
        self.window = window

    def setup(self, id):
        """
        Setups debug dialog

        :param id: debug id
        """
        self.window.debug[id] = QTreeView()
        self.window.debug[id].setRootIsDecorated(False)
        self.window.debug[id].setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.window.debug[id].setWordWrap(True)

        scroll = QScrollArea()
        scroll.setWidget(self.window.debug[id])
        scroll.setWidgetResizable(True)

        layout = QGridLayout()
        layout.addWidget(scroll, 1, 0)

        self.window.dialog['debug.' + id] = DebugDialog(self.window, id)
        self.window.dialog['debug.' + id].setLayout(layout)
        self.window.dialog['debug.' + id].setWindowTitle("Debug" + ": " + id)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.05 22:00:00                  #
# ================================================== #
import os


class AssistantsDebug:
    def __init__(self, window=None):
        """
        Assistants debug

        :param window: main window object
        """
        self.window = window
        self.id = 'assistants'

    def update(self):
        """Updates debug window."""
        self.window.debugger.begin(self.id)

        path = os.path.join(self.window.config.path, '', self.window.controller.assistant.assistants.config_file)
        self.window.debugger.add(self.id, 'File', path)

        # assistants
        assistants = self.window.controller.assistant.assistants.get_all()
        for key in assistants:
            prefix = "[{}] ".format(key)
            assistant = assistants[key]
            self.window.debugger.add(self.id, prefix + 'ID', str(key))
            self.window.debugger.add(self.id, 'id', str(assistant.id))
            self.window.debugger.add(self.id, 'name', str(assistant.name))
            self.window.debugger.add(self.id, 'description', str(assistant.description))
            self.window.debugger.add(self.id, 'model', str(assistant.model))
            self.window.debugger.add(self.id, 'instructions', str(assistant.instructions))
            self.window.debugger.add(self.id, 'meta', str(assistant.meta))
            self.window.debugger.add(self.id, 'tools', str(assistant.tools))
            self.window.debugger.add(self.id, 'files', str(assistant.files))

        self.window.debugger.end(self.id)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.08 22:00:00                  #
# ================================================== #

from PySide6.QtWidgets import QPlainTextEdit, QPushButton, QHBoxLayout, QLabel, QVBoxLayout

from ..widget.dialog import FileEditorDialog
from ...utils import trans


class Editor:
    def __init__(self, window=None):
        """
        Editor dialog

        :param window: main window object
        """
        self.window = window

    def setup(self):
        """Setups config editor dialog"""
        id = 'config'

        self.window.editor[id] = QPlainTextEdit()
        self.window.editor[id].setReadOnly(False)

        self.window.data['editor.btn.default'] = QPushButton(trans("dialog.editor.btn.defaults"))
        self.window.data['editor.btn.save'] = QPushButton(trans("dialog.editor.btn.save"))
        self.window.data['editor.btn.default'].clicked.connect(
            lambda: self.window.settings.load_default_editor())
        self.window.data['editor.btn.save'].clicked.connect(
            lambda: self.window.settings.save_editor())

        bottom_layout = QHBoxLayout()
        bottom_layout.addWidget(self.window.data['editor.btn.default'])
        bottom_layout.addWidget(self.window.data['editor.btn.save'])

        self.window.path_label[id] = QLabel("")
        self.window.path_label[id].setStyleSheet(self.window.controller.theme.get_style('text_bold'))

        self.window.data['dialog.editor.label'] = QLabel(trans('dialog.editor.label'))

        layout = QVBoxLayout()
        layout.addWidget(self.window.data['dialog.editor.label'])
        layout.addWidget(self.window.path_label[id])
        layout.addWidget(self.window.editor['config'])
        layout.addLayout(bottom_layout)

        self.window.dialog['config.editor'] = FileEditorDialog(self.window)
        self.window.dialog['config.editor'].setLayout(layout)
        self.window.dialog['config.editor'].setWindowTitle(trans('dialog.editor.title'))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.05 22:00:00                  #
# ================================================== #

class Audio:
    def __init__(self, window=None):
        """
        Audio/voice controller

        :param window: main window object
        """
        self.window = window

    def setup(self):
        """Setups controller"""
        self.update()

    def toggle_output(self):
        """Toggles audio/voice"""
        if self.window.controller.plugins.is_enabled('audio_azure'):
            self.disable_output()
        else:
            self.enable_output()

    def enable_output(self):
        """Enables audio/voice"""
        self.window.controller.plugins.enable('audio_azure')
        if self.window.controller.plugins.is_enabled('audio_azure') \
                and (self.window.controller.plugins.handler.plugins['audio_azure'].options['azure_api_key'] is None
                     or self.window.controller.plugins.handler.plugins['audio_azure'].options['azure_api_key'] == ''):
            self.window.ui.dialogs.alert("Azure API KEY is not set. Please set it in plugins settings.")
            self.window.controller.plugins.disable('audio_azure')
        self.window.config.save()
        self.update()

    def disable_output(self):
        """Disables audio/voice"""
        self.window.controller.plugins.disable('audio_azure')
        self.window.config.save()
        self.update()

    def update(self):
        """Updates UI"""
        self.update_menu()

    def update_menu(self):
        """Updates menu"""
        if self.window.controller.plugins.is_enabled('audio_azure'):
            self.window.menu['audio.output.azure'].setChecked(True)
        else:
            self.window.menu['audio.output.azure'].setChecked(False)

        if self.window.controller.plugins.is_enabled('audio_openai_tts'):
            self.window.menu['audio.output.tts'].setChecked(True)
        else:
            self.window.menu['audio.output.tts'].setChecked(False)

        if self.window.controller.plugins.is_enabled('audio_openai_whisper'):
            self.window.menu['audio.input.whisper'].setChecked(True)
        else:
            self.window.menu['audio.input.whisper'].setChecked(False)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.08 14:00:00                  #
# ================================================== #
import os
import sys

from PySide6.QtGui import QScreen
from PySide6.QtCore import QTimer, Signal, Slot
from PySide6.QtWidgets import (QApplication, QMainWindow)
from qt_material import QtStyleTools

from .config import Config
from .ui.main import UI
from .controller.main import Controller
from .debugger import Debug
from .settings import Settings
from .info import Info
from .gpt import Gpt
from .chain import Chain
from .context import Context
from .command import Command
from .image import Image
from .utils import get_init_value

from .plugin.self_loop.plugin import Plugin as SelfLoopPlugin
from .plugin.real_time.plugin import Plugin as RealTimePlugin
from .plugin.audio_azure.plugin import Plugin as AudioAzurePlugin
from .plugin.audio_openai_tts.plugin import Plugin as AudioOpenAITTSPlugin
from .plugin.audio_openai_whisper.plugin import Plugin as AudioOpenAIWhisperPlugin
from .plugin.cmd_web_google.plugin import Plugin as CmdWebGooglePlugin
from .plugin.cmd_files.plugin import Plugin as CmdFilesPlugin
from .plugin.cmd_code_interpreter.plugin import Plugin as CmdCodeInterpreterPlugin
from .plugin.cmd_custom.plugin import Plugin as CmdCustomCommandPlugin

from .llm.OpenAI import OpenAILLM
from .llm.AzureOpenAI import AzureOpenAILLM
from .llm.Anthropic import AnthropicLLM
from .llm.HuggingFace import HuggingFaceLLM
from .llm.Llama2 import Llama2LLM
from .llm.Ollama import OllamaLLM


class MainWindow(QMainWindow, QtStyleTools):
    statusChanged = Signal(str)

    def __init__(self):
        """App main window"""
        super().__init__()
        self.timer = None
        self.is_closing = False
        self.github = get_init_value("__github__")
        self.website = get_init_value("__website__")
        self.version = get_init_value("__version__")
        self.build = get_init_value("__build__")
        self.author = get_init_value("__author__")
        self.email = get_init_value("__email__")
        self.data = {}

        # setup config
        self.config = Config()
        self.config.init(True, True)
        self.context = Context(self.config)

        # app controller
        self.controller = Controller(self)
        self.debugger = Debug(self)
        self.info = Info(self)
        self.settings = Settings(self)
        self.command = Command(self)

        # handle config migration if needed
        self.controller.launcher.migrate_version()

        # load settings options from json
        self.controller.settings.load()

        # setup GPT, Langchain and DALL-E
        self.gpt = Gpt(self.config, self.context)
        self.chain = Chain(self.config, self.context)
        self.images = Image(self.config, self)

        # setup UI
        self.ui = UI(self)
        self.ui.setup()

        self.setWindowTitle('PYGPT.net v{} | build {}'.format(self.version, self.build))

        # setup signals
        self.statusChanged.connect(self.update_status)

    def log(self, data):
        """
        Logs data to console

        :param text: text to log
        """
        self.controller.debug.log(data)

    def set_theme(self, theme='dark_teal.xml', custom_css=None):
        """
        Updates material theme and applies custom CSS

        :param theme: Material theme name
        :param custom_css: custom CSS file
        """
        inverse = False
        extra = {
            'density_scale': self.config.data['layout.density'],
            'pyside6': True,
        }
        self.apply_stylesheet(self, theme, invert_secondary=inverse, extra=extra)

        # append custom CSS
        if custom_css is not None:
            path = os.path.join(self.config.get_root_path(), 'data', 'css', custom_css)
            stylesheet = self.styleSheet()
            if os.path.exists(path):
                with open(path) as file:
                    self.setStyleSheet(stylesheet + file.read().format(**os.environ))

    def add_plugin(self, plugin):
        """
        Adds plugin

        :param plugin: plugin instance
        """
        plugin.attach(self)
        self.controller.plugins.register(plugin)

    def add_llm(self, llm):
        """
        Adds Langchain LLM wrapper

        :param llm: llm instance
        """
        id = llm.id
        self.chain.register(id, llm)

    def setup(self):
        """Setups app"""
        self.controller.setup()
        self.controller.plugins.setup()
        self.controller.setup_plugins()
        self.timer = QTimer()
        self.timer.timeout.connect(self.update)
        self.timer.start(30)

    def post_setup(self):
        """Called after setup"""
        self.controller.layout.post_setup()

    def update(self):
        """Called on update"""
        self.debugger.update()
        self.controller.update()

    def set_status(self, text):
        """
        Updates status text

        :param text: status text
        """
        self.data['status'].setText(str(text))

    @Slot(str)
    def update_status(self, text):
        self.set_status(text)

    def closeEvent(self, event):
        """
        Handles close event

        :param event: close event
        """
        self.is_closing = True
        print("Closing...")
        print("Sending terminate signal to plugins...")
        self.controller.plugins.destroy()
        print("Saving notepad...")
        self.controller.notepad.save()
        print("Saving layout state...")
        self.controller.layout.save()
        print("Stopping event loop...")
        self.timer.stop()
        print("Saving config...")
        self.config.save()
        print("Saving presets...")
        self.config.save_presets()
        print("Exiting...")
        event.accept()  # let the window close


class Launcher:
    def __init__(self):
        """Launcher"""
        self.app = None
        self.window = None

    def init(self):
        """Initializes app"""
        self.app = QApplication(sys.argv)
        self.window = MainWindow()
        self.app.aboutToQuit.connect(self.app.quit)

    def add_plugin(self, plugin=None):
        """
        Registers plugin

        :param plugin: plugin instance
        """
        self.window.add_plugin(plugin)

    def add_llm(self, llm=None):
        """
        Registers LLMs

        :param llm: llm instance
        """
        self.window.add_llm(llm)

    def run(self):
        """Runs app"""
        margin = 50
        self.window.setup()
        available_geometry = self.window.screen().availableGeometry()
        pos = QScreen.availableGeometry(QApplication.primaryScreen()).topLeft()
        self.window.resize(available_geometry.width() - margin, available_geometry.height() - margin)
        self.window.show()
        self.window.move(pos)
        self.window.post_setup()
        try:
            sys.exit(self.app.exec())
        except SystemExit:
            print("Closing...")


def run():
    """Runs app"""
    # initialize app
    launcher = Launcher()
    launcher.init()

    # register plugins
    launcher.add_plugin(SelfLoopPlugin())
    launcher.add_plugin(RealTimePlugin())
    launcher.add_plugin(AudioAzurePlugin())
    launcher.add_plugin(AudioOpenAITTSPlugin())
    launcher.add_plugin(AudioOpenAIWhisperPlugin())
    launcher.add_plugin(CmdWebGooglePlugin())
    launcher.add_plugin(CmdFilesPlugin())
    launcher.add_plugin(CmdCodeInterpreterPlugin())
    launcher.add_plugin(CmdCustomCommandPlugin())

    # register langchain LLMs
    launcher.add_llm(OpenAILLM())
    launcher.add_llm(AzureOpenAILLM())
    launcher.add_llm(AnthropicLLM())
    launcher.add_llm(HuggingFaceLLM())
    launcher.add_llm(Llama2LLM())
    launcher.add_llm(OllamaLLM())

    # run app
    launcher.run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.05 22:00:00                  #
# ================================================== #

import datetime
import os
import requests
from openai import OpenAI


class Image:
    DIRNAME = "img"

    def __init__(self, config, window):
        """
        DALL-E Wrapper

        :param config: Config object
        """
        self.config = config
        self.window = window
        if not self.config.initialized:
            self.config.init()

    def init(self):
        """Initialize OpenAI API key"""
        pass

    def generate(self, prompt, model="dall-e-3", num=None):
        """
        Calls DALL-E API

        :param prompt: Prompt
        :param model: Model name
        :param num: Number of variants
        :return: Images paths list
        """
        if not self.window.config.data['img_raw']:
            img_prompt = '''
            # Rules
            1. Apply these rules if the request is related to image generation or image description; otherwise, relay the user's prompt as is.
            2. Translate any non-English image prompts accurately into English, ensuring the essence of the original request is maintained for DALL-E's comprehension.
            3. Transform the translated prompt into a precise and specific instruction that DALL-E can unambiguously understand, focusing solely on describing the desired image.
            4. Preface your instruction with "A photograph of..." or "An image of..." to indicate a visual output request, refraining from using phrases like "Please generate..." or simply "Generate...".
            5. Emphasize the critical keywords and pertinent details within the prompt to assist DALL-E in producing an optimal image response.
            6. For prompts centered on intricate concepts or specialized topics, supply supplementary context to better inform DALL-E's interpretation and creativity.
            7. Designate the desired level of abstraction, realistic depiction, or a particular art style as part of the query to guide the image generation process.
            8. Set explicit parameters such as artistic style, composition, and any additional defining traits for the cognitive synthesis, tailored to enhance the final visual product.
            9. Compile and incorporate an exhaustive inventory of elements and nuances to achieve the most detailed and sophisticated image outcome possible.
            10. Return the enhanced, detail-rich description tailored for DALL-E, omitting any extraneous commentary or instruction.
            11. Use as many details as possible to describe the image.
            12. Use all possible different DESCENTS with EQUAL probability.
            '''

            # call GPT for generate best prompt
            response = self.window.gpt.quick_call(prompt, img_prompt, False, 200,
                                                  self.window.config.data['img_prompt_model'], 2.0)  # max creativity
            if response is not None and response != "":
                prompt = response

        print("Generating from: '{}'...".format(prompt))

        client = OpenAI(
            # This is the default and can be omitted
            api_key=self.config.data["api_key"],
            organization=self.config.data["organization_key"],
        )

        response = client.images.generate(
            model=model,
            prompt=prompt,
            n=num,
            size=self.config.data["img_resolution"],
        )

        # generate and download images
        paths = []
        for i in range(num):
            url = response.data[i].url
            res = requests.get(url)
            name = self.make_safe_filename(prompt) + "-" + datetime.date.today().strftime(
                "%Y-%m-%d") + "_" + datetime.datetime.now().strftime("%H-%M-%S") + "-" + str(i + 1) + ".png"
            path = os.path.join(self.config.path, self.DIRNAME, name)
            print("Downloading... [" + str(i + 1) + " of " + str(num) + "] to: " + path)
            open(path, "wb").write(res.content)
            paths.append(path)

        return paths, prompt

    def make_safe_filename(self, name):
        """
        Makes safe filename

        :param name: Filename to make safe
        :return: Safe filename
        """

        def safe_char(c):
            if c.isalnum():
                return c
            else:
                return "_"

        return "".join(safe_char(c) for c in name).rstrip("_")[:30]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.10 13:00:00                  #
# ================================================== #
import json
import os

from PySide6.QtGui import QAction

from ..utils import trans


class Theme:
    def __init__(self, window=None):
        """
        Theme controller

        :param window: main window object
        """
        self.window = window
        self.css = {}

    def load_css(self):
        """Loads CSS"""
        path = os.path.join(self.window.config.get_root_path(), 'data', 'css', 'highlighter.json')
        if os.path.exists(path):
            try:
                with open(path, 'r') as f:
                    self.css['highlighter'] = json.load(f)
                    f.close()
            except Exception as e:
                print(e)

    def get_css(self, name):
        """
        Returns CSS

        :param name: CSS name
        :return: CSS
        """
        if name in self.css:
            return self.css[name]
        return {}

    def toggle(self, name):
        """
        Toggles theme

        :param name: theme name
        """
        self.window.config.data['theme'] = name
        self.window.config.save()
        self.apply()
        self.window.set_theme(name + '.xml', 'style.css')  # style.css = additional custom stylesheet
        self.update()

    def apply(self):
        """Applies theme"""
        # windows
        self.window.data['output'].setStyleSheet(self.get_style('chat_output'))
        self.window.data['input'].setStyleSheet(self.get_style('chat_input'))
        self.window.data['ctx.contexts'].setStyleSheet(self.get_style('ctx.contexts'))
        # notepads
        self.window.data['notepad1'].setStyleSheet(self.get_style('chat_output'))
        self.window.data['notepad2'].setStyleSheet(self.get_style('chat_output'))
        self.window.data['notepad3'].setStyleSheet(self.get_style('chat_output'))
        self.window.data['notepad4'].setStyleSheet(self.get_style('chat_output'))
        self.window.data['notepad5'].setStyleSheet(self.get_style('chat_output'))

        # apply to syntax highlighter
        self.apply_syntax_highlighter(self.window.config.data['theme'])

    def get_style(self, element):
        """
        Returns style for element

        :param element: element name
        :return: style
        """
        # get theme element style
        if  element == "chat_output":
            return 'font-size: {}px;'.format(self.window.config.data['font_size'])
        elif element == "chat_input":
            return 'font-size: {}px;'.format(self.window.config.data['font_size.input'])
        elif element == "ctx.contexts":
            return 'font-size: {}px;'.format(self.window.config.data['font_size.ctx'])
        elif element == "text_bold":
            return "font-weight: bold;"
        elif element == "text_small":
            return "font-size: 9px;"

    def apply_syntax_highlighter(self, theme):
        """Applies syntax highlight"""
        self.window.data['output_highlighter'].setTheme(self.get_css('highlighter'))

    def update(self):
        """Updates theme menu"""
        for theme in self.window.menu['theme']:
            self.window.menu['theme'][theme].setChecked(False)
        current = self.window.config.data['theme']
        if current in self.window.menu['theme']:
            self.window.menu['theme'][current].setChecked(True)

    def get_themes_list(self):
        """
        Returns list of themes

        :return: list of themes
        """
        return ['dark_amber',
                'dark_blue',
                'dark_cyan',
                'dark_lightgreen',
                'dark_pink',
                'dark_purple',
                'dark_red',
                'dark_teal',
                'dark_yellow',
                # 'light_amber',
                # 'light_blue',
                # 'light_cyan',
                # 'light_cyan_500',
                # 'light_lightgreen',
                # 'light_pink',
                # 'light_purple',
                # 'light_red',
                # 'light_teal',
                # 'light_yellow'
        ]

    def trans_theme(self, theme):
        """
        Translates theme name

        :param theme: theme name
        :return: translated theme name
        """
        return theme.replace('_', ' ').title().replace('Dark ', trans('theme.dark') + ': ').replace('Light ', trans(
            'theme.light') + ': ')

    def setup(self):
        """Setups theme"""
        # load css files
        self.load_css()

        # setup menu
        themes = self.get_themes_list()
        for theme in themes:
            name = self.trans_theme(theme)
            self.window.menu['theme'][theme] = QAction(name, self.window, checkable=True)
            self.window.menu['theme'][theme].triggered.connect(
                lambda checked=None, theme=theme: self.window.controller.theme.toggle(theme))
            self.window.menu['menu.theme'].addAction(self.window.menu['theme'][theme])

        # apply theme
        theme = self.window.config.data['theme']
        self.toggle(theme)

    def reload(self):
        """Reloads theme"""
        theme = self.window.config.data['theme']
        self.toggle(theme)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.07 10:00:00                  #
# ================================================== #
import json

from PySide6.QtGui import QTextCursor
from PySide6.QtWidgets import QApplication

from ..context import ContextItem
from ..history import History
from ..utils import trans


class Input:
    def __init__(self, window=None):
        """
        Input controller

        :param window: main window
        """
        self.window = window
        self.history = History(self.window.config)
        self.locked = False
        self.force_stop = False

    def setup(self):
        """Sets up input"""

        # stream
        if self.window.config.data['stream']:
            self.window.data['input.stream'].setChecked(True)
        else:
            self.window.data['input.stream'].setChecked(False)

        # send clear
        if self.window.config.data['send_clear']:
            self.window.data['input.send_clear'].setChecked(True)
        else:
            self.window.data['input.send_clear'].setChecked(False)

        # send with enter/shift/disabled
        if self.window.config.data['send_mode'] == 2:
            self.window.data['input.send_shift_enter'].setChecked(True)
            self.window.data['input.send_enter'].setChecked(False)
            self.window.data['input.send_none'].setChecked(False)
        elif self.window.config.data['send_mode'] == 1:
            self.window.data['input.send_enter'].setChecked(True)
            self.window.data['input.send_shift_enter'].setChecked(False)
            self.window.data['input.send_none'].setChecked(False)
        elif self.window.config.data['send_mode'] == 0:
            self.window.data['input.send_enter'].setChecked(False)
            self.window.data['input.send_shift_enter'].setChecked(False)
            self.window.data['input.send_none'].setChecked(True)

        # cmd enabled
        if self.window.config.data['cmd']:
            self.window.data['cmd.enabled'].setChecked(True)
        else:
            self.window.data['cmd.enabled'].setChecked(False)

        # set focus to input
        self.window.data['input'].setFocus()

    def toggle_stream(self, value):
        """
        Toggles stream

        :param value: value of the checkbox
        """
        self.window.config.data['stream'] = value

    def toggle_cmd(self, value):
        """
        Toggles cmd enabled

        :param value: value of the checkbox
        """
        self.window.config.data['cmd'] = value

    def toggle_send_clear(self, value):
        """
        Toggles send clear

        :param value: value of the checkbox
        """
        self.window.config.data['send_clear'] = value

    def toggle_send_shift(self, value):
        """
        Toggles send with shift

        :param value: value of the checkbox
        """
        self.window.config.data['send_mode'] = value

    def lock_input(self):
        """
        Locks input
        """
        self.locked = True
        self.window.data['input.send_btn'].setEnabled(False)
        self.window.data['input.stop_btn'].setVisible(True)

    def unlock_input(self):
        """
        Unlocks input
        """
        self.locked = False
        self.window.data['input.send_btn'].setEnabled(True)
        self.window.data['input.stop_btn'].setVisible(False)

    def stop(self):
        """
        Stops input
        """
        self.window.controller.assistant.force_stop = True
        self.force_stop = True
        self.window.gpt.stop()
        self.unlock_input()

    def send_text(self, text):
        """
        Sends text to GPT

        :param text: text to send
        """
        self.window.statusChanged.emit(trans('status.sending'))

        # prepare names
        self.window.log("User name: {}".format(self.window.config.data['user_name']))  # log
        self.window.log("AI name: {}".format(self.window.config.data['ai_name']))  # log

        user_name = self.window.controller.plugins.apply('user.name', self.window.config.data['user_name'])
        ai_name = self.window.controller.plugins.apply('ai.name', self.window.config.data['ai_name'])

        self.window.log("User name [after plugin: user_name]: {}".format(self.window.config.data['user_name']))  # log
        self.window.log("AI name [after plugin: ai_name]: {}".format(self.window.config.data['ai_name']))  # log

        # store history (input)
        if self.window.config.data['store_history'] and text is not None and text.strip() != "":
            self.history.save(text)

        # get mode
        mode = self.window.config.data['mode']

        # clear
        self.window.gpt.file_ids = []  # file ids

        # upload new attachments if assistant mode
        if mode == 'assistant':
            try:
                # it uploads only new attachments (not uploaded before to remote)
                attachments = self.window.controller.attachment.attachments.get_all(mode)
                if len(attachments) > 0:
                    self.window.set_status(trans('status.uploading'))
                self.window.controller.assistant.upload_attachments(mode, attachments)
                self.window.gpt.file_ids = self.window.controller.attachment.attachments.get_ids(mode)

            except Exception as e:
                print(e)
                self.window.ui.dialogs.alert(str(e))
            self.window.set_status(trans('status.uploaded'))

            # create or get current thread, it is required here
            if self.window.config.data['assistant_thread'] is None:
                try:
                    self.window.config.data['assistant_thread'] = self.window.controller.assistant.create_thread()
                except Exception as e:
                    print(e)
                    self.window.ui.dialogs.alert(str(e))

        # create ctx item
        ctx = ContextItem()
        ctx.mode = mode
        ctx.set_input(text, user_name)
        ctx.set_output(None, ai_name)

        # store thread id, assistant id and pass to gpt wrapper
        if mode == 'assistant':
            ctx.thread = self.window.config.data['assistant_thread']
            self.window.gpt.assistant_id = self.window.config.data['assistant']
            self.window.gpt.thread_id = ctx.thread

        # log
        self.window.log("Context: input: {}".format(ctx.dump()))

        # apply plugins
        ctx = self.window.controller.plugins.apply('ctx.before', ctx)

        # log
        self.window.log("Context: input [after plugin: ctx.before]: {}".format(ctx.dump()))
        self.window.log("System: {}".format(self.window.gpt.system_prompt))

        # apply cfg, plugins
        self.window.gpt.user_name = ctx.input_name
        self.window.gpt.ai_name = ctx.output_name
        self.window.chain.user_name = ctx.input_name
        self.window.chain.ai_name = ctx.output_name

        # prepare system prompt
        sys_prompt = self.window.config.data['prompt']
        sys_prompt = self.window.controller.plugins.apply('system.prompt', sys_prompt)

        # if commands enabled: append commands prompt
        if self.window.config.data['cmd']:
            sys_prompt += " " + self.window.command.get_prompt()
            sys_prompt = self.window.gpt.system_prompt = self.window.controller.plugins.apply('cmd.syntax', sys_prompt)

        # set system prompt
        self.window.gpt.system_prompt = sys_prompt
        self.window.chain.system_prompt = sys_prompt

        # log
        self.window.log("System [after plugin: system.prompt]: {}".format(self.window.gpt.system_prompt))
        self.window.log("User name: {}".format(self.window.gpt.user_name))
        self.window.log("AI name: {}".format(self.window.gpt.ai_name))
        self.window.log("Appending input to chat window...")

        # append input to chat window
        self.window.controller.output.append_input(ctx)
        QApplication.processEvents()  # process events to update UI
        self.window.controller.ui.update()  # update UI

        # async or sync mode
        stream_mode = self.window.config.data['stream']

        # disable stream mode for vision mode (tmp)
        if mode == "vision":
            stream_mode = False

        # call the model
        try:
            # set attachments (attachments are separated by mode)
            self.window.gpt.attachments = self.window.controller.attachment.attachments.get_all(mode)

            # make API call
            try:
                # lock input
                self.lock_input()

                if mode == "langchain":
                    self.window.log("Calling LangChain...")  # log
                    ctx = self.window.chain.call(text, ctx, stream_mode)
                else:
                    self.window.log("Calling OpenAI API...")  # log
                    ctx = self.window.gpt.call(text, ctx, stream_mode)

                    if mode == 'assistant':

                        # get run ID and save it in ctx
                        self.window.gpt.context.append_run(ctx.run_id)

                        # handle assistant run
                        self.window.controller.assistant.handle_run(ctx)

                if ctx is not None:
                    self.window.log("Context: output: {}".format(ctx.dump()))  # log
                else:
                    # error in call if ctx is None
                    self.window.log("Context: output: None")
                    self.window.ui.dialogs.alert(trans('status.error'))
                    self.window.set_status(trans('status.error'))

            except Exception as e:
                self.window.log("GPT output error: {}".format(e))  # log
                print("Error in send text (GPT call): " + str(e))
                self.window.ui.dialogs.alert(str(e))
                self.window.set_status(trans('status.error'))

            # handle response (if no assistant mode)
            if mode != "assistant":
                self.handle_response(ctx, mode, stream_mode)

        except Exception as e:
            self.window.log("Output error: {}".format(e))  # log
            print("Error in send text: " + str(e))
            self.window.ui.dialogs.alert(str(e))
            self.window.set_status(trans('status.error'))

        # if commands enabled: post-execute commands (if no assistant mode)
        if mode != "assistant":
            self.handle_commands(ctx)
            self.unlock_input()

        # handle ctx name (generate name if not initialized)
        if self.window.config.data['ctx.auto_summary']:
            self.handle_ctx_name(ctx)

        return ctx

    def handle_ctx_name(self, ctx):
        """
        Handle context name (summarize input and output)
        :param ctx: ContextItem
        """
        if ctx is not None:
            if not self.window.gpt.context.is_ctx_initialized():
                current = self.window.gpt.context.current_ctx
                title = self.window.gpt.prepare_ctx_name(ctx)
                if title is not None and title != "":
                    self.window.controller.context.update_name(current, title)

    def handle_commands(self, ctx):
        """
        Handle plugin commands

        :param ctx: ContextItem
        """
        if ctx is not None and self.window.config.data['cmd']:
            cmds = self.window.command.extract_cmds(ctx.output)
            self.window.log("Executing commands...")
            self.window.set_status("Executing commands...")
            ctx = self.window.controller.plugins.apply_cmds(ctx, cmds)
            self.window.set_status("")

    def handle_response(self, ctx, mode, stream_mode=False):
        """
        Handle response from LLM

        :param ctx: ContextItem
        :param mode: Mode
        :param stream_mode: Async stream mode
        """
        # if async stream mode
        if stream_mode and mode != 'assistant':
            output = ""
            output_tokens = 0
            begin = True
            submode = None  # submode for langchain (chat, completion)

            # get submode for langchain
            if mode == "langchain":
                cfg = self.window.config.get_model_cfg(self.window.config.data['model'])
                submode = 'chat'
                # get available modes for langchain
                if 'langchain' in cfg:
                    if 'chat' in cfg['langchain']['mode']:
                        submode = 'chat'
                    elif 'completion' in cfg['langchain']['mode']:
                        submode = 'completion'

            # read stream
            try:
                self.window.log("Reading stream...")  # log
                for chunk in ctx.stream:
                    # if force stop then break
                    if self.force_stop:
                        break

                    response = None
                    if mode == "chat" or mode == "vision" or mode == "assistant":
                        if chunk.choices[0].delta.content is not None:
                            response = chunk.choices[0].delta.content
                    elif mode == "completion":
                        if chunk.choices[0].text is not None:
                            response = chunk.choices[0].text

                    # langchain can provide different modes itself
                    elif mode == "langchain":
                        if submode == 'chat':
                            # if chat model response is object
                            if chunk.content is not None:
                                response = chunk.content
                        elif submode == 'completion':
                            # if completion response is string
                            if chunk is not None:
                                response = chunk

                    if response is not None:
                        # prevent empty begin
                        if begin and response == "":
                            continue
                        output += response
                        output_tokens += 1
                        self.window.controller.output.append_chunk(ctx, response, begin)
                        QApplication.processEvents()  # process events to update UI
                        self.window.controller.ui.update()  # update UI
                        begin = False

            except Exception as e:
                # debug
                # self.window.log("Stream error: {}".format(e))  # log
                # print("Error in stream: " + str(e))
                # self.window.ui.dialogs.alert(str(e))
                pass

            self.window.controller.output.append("\n")  # append EOL
            self.window.log("End of stream.")  # log

            # update ctx
            if ctx is not None:
                ctx.output = output
                ctx.set_tokens(ctx.input_tokens, output_tokens)

            # --- end of stream mode ---

        # apply plugins
        if ctx is not None:
            ctx = self.window.controller.plugins.apply('ctx.after', ctx)

        # log
        if ctx is not None:
            self.window.log("Context: output [after plugin: ctx.after]: {}".format(ctx.dump()))
            self.window.log("Appending output to chat window...")

            # only append output if not in async stream mode, TODO: plugin output add
            if not stream_mode:
                self.window.controller.output.append_output(ctx)

            self.handle_complete(ctx)

    def handle_complete(self, ctx):
        """Handles completed context"""
        # save context
        self.window.gpt.context.store()
        self.window.set_status(
            trans('status.tokens') + ": {} + {} = {}".format(ctx.input_tokens, ctx.output_tokens, ctx.total_tokens))

        # store history (output)
        if self.window.config.data['store_history'] and ctx.output is not None and ctx.output.strip() != "":
            self.history.save(ctx.output)

    def user_send(self, text=None):
        """Sends real user input"""
        text = self.window.controller.plugins.apply('user.send', text)
        self.send(text)

    def send(self, text=None):
        """
        Sends input text to API
        """

        # check if input is locked
        if self.locked:
            return

        mode = self.window.config.data['mode']

        if mode == 'assistant':
            if self.window.config.data['assistant'] is None or self.window.config.data['assistant'] == "":
                self.window.ui.dialogs.alert(trans('error.assistant_not_selected'))
                return
        elif mode == 'vision':
            if self.window.controller.camera.is_enabled():
                if self.window.controller.camera.is_auto():
                    self.window.controller.camera.capture_frame(False)

        # unlock run thread if locked
        self.window.controller.assistant.force_stop = False
        self.force_stop = False
        self.window.statusChanged.emit(trans('status.sending'))

        ctx = None
        if text is None:
            text = self.window.data['input'].toPlainText().strip()

        self.window.log("Input text: {}".format(text))  # log
        text = self.window.controller.plugins.apply('input.before', text)

        self.window.log("Input text [after plugin: input.before]: {}".format(text))  # log

        if len(text.strip()) > 0 or (mode == 'vision' and self.window.controller.attachment.has_attachments(mode)):
            if self.window.config.data['send_clear']:
                self.window.data['input'].clear()

            # check API key
            if self.window.config.data['api_key'] is None or self.window.config.data['api_key'] == '':
                self.window.controller.launcher.show_api_monit()
                self.window.controller.ui.update()
                self.window.statusChanged.emit("")
                return

            # init api key if defined later
            self.window.gpt.init()
            self.window.images.init()

            # prepare context, create new if no contexts (first run)
            if len(self.window.gpt.context.contexts) == 0:
                self.window.gpt.context.new()
                self.window.controller.context.update()
                self.window.log("New context created...")  # log

            # process events to update UI
            QApplication.processEvents()

            # send to API
            if self.window.config.data['mode'] == 'img':
                ctx = self.window.controller.image.send_text(text)
            else:
                ctx = self.window.controller.input.send_text(text)
        else:
            self.window.statusChanged.emit("")

        # clear attachments after send if enabled
        if self.window.config.data['attachments_send_clear']:
            self.window.controller.attachment.clear(True)
            self.window.controller.attachment.update()

        if ctx is not None:
            self.window.log("Context: output: {}".format(ctx.dump()))  # log
            ctx = self.window.controller.plugins.apply('ctx.end', ctx)  # apply plugins
            self.window.log("Context: output [after plugin: ctx.end]: {}".format(ctx.dump()))  # log
            self.window.controller.ui.update()  # update UI

            # if reply from commands then send reply (as response JSON)
            if ctx.reply:
                self.window.controller.input.send(json.dumps(ctx.results))
                self.window.controller.ui.update()
            return

        self.window.controller.ui.update()  # update UI

    def append(self, text):
        """
        Appends text to input

        :param text: text to append
        """
        cur = self.window.data['input'].textCursor()  # Move cursor to end of text
        cur.movePosition(QTextCursor.End)
        s = str(text) + "\n"
        while s:
            head, sep, s = s.partition("\n")  # Split line at LF
            cur.insertText(head)  # Insert text at cursor
            if sep:  # New line if LF
                cur.insertBlock()
        self.window.data['input'].setTextCursor(cur)  # Update visible cursor



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.10 13:00:00                  #
# ================================================== #

from urllib.request import urlopen, Request
from packaging.version import parse as parse_version
import os
import shutil
import json
import ssl
from .utils import trans


class Updater:
    def __init__(self, window=None):
        """
        Updater (patcher)

        :param window: main window
        """
        self.window = window

    def check(self):
        """Checks for updates"""
        print("Checking for updates...")
        url = self.window.website + "/api/version?v=" + str(self.window.version)
        try:
            ctx = ssl.create_default_context()
            ctx.check_hostname = False
            ctx.verify_mode = ssl.CERT_NONE

            req = Request(
                url=url,
                headers={'User-Agent': 'Mozilla/5.0'}
            )
            response = urlopen(req, context=ctx, timeout=3)
            data_json = json.loads(response.read())
            newest_version = data_json["version"]
            newest_build = data_json["build"]

            # changelog
            changelog = ""
            if "changelog" in data_json:
                changelog = data_json["changelog"]

            parsed_newest_version = parse_version(newest_version)
            parsed_current_version = parse_version(self.window.version)
            if parsed_newest_version > parsed_current_version:
                self.show_version_dialog(newest_version, newest_build, changelog)
            else:
                print("No updates available")
        except Exception as e:
            print("Failed to check for updates")
            print(e)

    def show_version_dialog(self, version, build, changelog):
        """
        Displays new version dialog

        :param version: version number
        :param build: build date
        :param changelog: changelog
        """
        txt = trans('update.new_version') + ": " + str(version) + " (" + trans('update.released') + ": " + str(
            build) + ")"
        txt += "\n" + trans('update.current_version') + ": " + self.window.version
        self.window.dialog['update'].changelog.setPlainText(changelog)
        self.window.dialog['update'].message.setText(txt)
        self.window.ui.dialogs.open('update')

    def patch(self):
        """Patch config files to current version"""
        try:
            self.patch_config()
            self.patch_models()
            self.patch_presets()
            # TODO: add context patcher
        except Exception as e:
            print("Failed to patch config files!")
            print(e)

    def patch_dir(self, dirname="", force=False):
        """
        Patches directory
        :param dirname: Directory name
        :param force: Force update
        """
        try:
            # dir
            dst_dir = os.path.join(self.window.config.path, dirname)
            src = os.path.join(self.window.config.get_root_path(), 'data', 'config', dirname)
            for file in os.listdir(src):
                src_file = os.path.join(src, file)
                dst_file = os.path.join(dst_dir, file)
                if not os.path.exists(dst_file) or force:
                    shutil.copyfile(src_file, dst_file)
        except Exception as e:
            print(e)

    def patch_file(self, filename="", force=False):
        """
        Patches file
        :param filename: File name
        :param force: Force update
        """
        try:
            # file
            dst = os.path.join(self.window.config.path, filename)
            if not os.path.exists(dst) or force:
                src = os.path.join(self.window.config.get_root_path(), 'data', 'config', filename)
                shutil.copyfile(src, dst)
        except Exception as e:
            print(e)

    def patch_models(self):
        """Migrates models to current version"""
        data = self.window.config.models
        version = "0.0.0"
        updated = False
        if '__meta__' in data and 'version' in data['__meta__']:
            version = data['__meta__']['version']
        old = parse_version(version)
        current = parse_version(self.window.version)
        if old < current:
            if old < parse_version("2.0.1"):
                self.patch_file('models.json', True)
                self.window.config.load_models()
                data = self.window.config.models
                updated = True
            if old < parse_version("0.9.1"):
                # apply meta only (not attached in 0.9.0)
                updated = True

        # update file
        if updated:
            data = dict(sorted(data.items()))
            self.window.config.models = data
            self.window.config.save_models()
            print("Migrated models.json.")

    def patch_presets(self):
        """Migrates presets to current version"""
        for k in self.window.config.presets:
            data = self.window.config.presets[k]
            version = "0.0.0"
            updated = False
            if '__meta__' in data and 'version' in data['__meta__']:
                version = data['__meta__']['version']
            old = parse_version(version)
            current = parse_version(self.window.version)
            if old < current:
                if old < parse_version("2.0.0"):
                    self.patch_file('presets', True)

            # update file
            if updated:
                data = dict(sorted(data.items()))
                self.window.config.presets[k] = data
                self.window.config.save_preset(k)
                print("Migrated presets.")

    def patch_config(self):
        """Migrates config to current version"""
        data = self.window.config.data
        version = "0.0.0"
        updated = False
        if '__meta__' in data and 'version' in data['__meta__']:
            version = data['__meta__']['version']
        old = parse_version(version)
        current = parse_version(self.window.version)
        if old < current:
            if old < parse_version("2.0.16"):
                if 'vision.capture.idx' not in data:
                    data['vision.capture.idx'] = 0
                if 'img_raw' not in data:
                    data['img_raw'] = True
                if 'img_prompt_model' not in data:
                    data['img_prompt_model'] = "gpt-4-1106-preview"
                updated = True
            if old < parse_version("2.0.14"):
                if 'vision.capture.enabled' not in data:
                    data['vision.capture.enabled'] = True
                if 'vision.capture.auto' not in data:
                    data['vision.capture.auto'] = True
                if 'vision.capture.width' not in data:
                    data['vision.capture.width'] = 800
                if 'vision.capture.height' not in data:
                    data['vision.capture.height'] = 600
                updated = True
            if old < parse_version("2.0.13"):
                if 'layout.density' not in data:
                    data['layout.density'] = 0
                else:
                    if data['layout.density'] == -2:
                        data['layout.density'] = 0
                updated = True
            if old < parse_version("2.0.8"):
                if 'plugins' not in data:
                    data['plugins'] = {}
                if 'cmd_web_google' not in data['plugins']:
                    data['plugins']['cmd_web_google'] = {}
                data['plugins']['cmd_web_google']['prompt_summarize'] = "Summarize the English text in a maximum of 3 " \
                                                                        "paragraphs, trying to find the most " \
                                                                        "important content that can help answer the " \
                                                                        "following question: "
                data['plugins']['cmd_web_google']['chunk_size'] = 100000
                data['plugins']['cmd_web_google']['max_page_content_length'] = 0
                updated = True
            if old < parse_version("2.0.6"):
                if 'layout.density' not in data:
                    data['layout.density'] = -2
                updated = True
            if old < parse_version("2.0.1"):
                if 'send_mode' not in data:
                    data['send_mode'] = 1
                if 'send_shift_enter' in data:
                    del data['send_shift_enter']
                if 'font_size.input' not in data:
                    data['font_size.input'] = 11
                if 'font_size.ctx' not in data:
                    data['font_size.ctx'] = 9
                if 'ctx.auto_summary' not in data:
                    data['ctx.auto_summary'] = True
                if 'ctx.auto_summary.system' not in data:
                    data['ctx.auto_summary.system'] = "You are an expert in conversation summarization"
                if 'ctx.auto_summary.prompt' not in data:
                    data['ctx.auto_summary.prompt'] = "Summarize topic of this conversation in one sentence. Use best " \
                                                      "keywords to describe it. Summary must be in the same language " \
                                                      "as the conversation and it will be used for conversation title " \
                                                      "so it must be EXTREMELY SHORT and concise - use maximum 5 " \
                                                      "words: \n\nUser: {input}\nAI Assistant: {output}"
                updated = True
            if old < parse_version("2.0.0"):
                data['theme'] = 'dark_teal'  # force, because removed light themes!
                if 'cmd' not in data:
                    data['cmd'] = True
                if 'stream' not in data:
                    data['stream'] = True
                if 'attachments_send_clear' not in data:
                    data['attachments_send_clear'] = True
                if 'assistant' not in data:
                    data['assistant'] = None
                if 'assistant_thread' not in data:
                    data['assistant_thread'] = None
                updated = True
            if old < parse_version("0.9.6"):
                print("Migrating config from < 0.9.6...")
                data['debug'] = True  # enable debug by default
                updated = True
            if old < parse_version("0.9.4"):
                print("Migrating config from < 0.9.4...")
                if 'plugins' not in data:
                    data['plugins'] = {}
                if 'plugins_enabled' not in data:
                    data['plugins_enabled'] = {}
                updated = True
            if old < parse_version("0.9.2"):
                print("Migrating config from < 0.9.2...")
                keys_to_remove = ['ui.ctx.min_width',
                                  'ui.ctx.max_width',
                                  'ui.toolbox.min_width',
                                  'ui.toolbox.max_width',
                                  'ui.dialog.settings.width',
                                  'ui.dialog.settings.height',
                                  'ui.chatbox.font.color']
                for key in keys_to_remove:
                    if key in data:
                        del data[key]
                if 'theme' not in data:
                    data['theme'] = "dark_teal"
                updated = True

            if old < parse_version("0.9.1"):
                print("Migrating config from < 0.9.1...")
                keys_to_remove = ['user_id', 'custom']  # not needed anymore
                for key in keys_to_remove:
                    if key in data:
                        del data[key]
                keys_to_add = ['organization_key']
                for key in keys_to_add:
                    if key not in data:
                        data[key] = ""
                updated = True

        # update file
        if updated:
            data = dict(sorted(data.items()))
            self.window.config.data = data
            self.window.config.save()
            print("Migrated config.json.")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.05 22:00:00                  #
# ================================================== #

from datetime import datetime
from PySide6.QtGui import QTextCursor


class Output:
    def __init__(self, window=None):
        """
        Output controller

        :param window: main window
        """
        self.window = window

    def setup(self):
        """Setups output"""
        self.window.data['output.timestamp'].setChecked(self.window.config.data['output_timestamp'])

    def clear(self):
        """
        Clears output
        """
        self.window.data['output'].clear()

    def append_context(self):
        """
        Appends context to output
        """
        for item in self.window.gpt.context.items:
            self.append_context_item(item)

    def append_context_item(self, item):
        """
        Appends context item to output

        :param item: context item
        """
        self.append_input(item)
        self.append_output(item)

    def append_input(self, item):
        """
        Appends input to output

        :param item: context item
        """
        if item.input is None or item.input == "":
            return
        if self.window.config.data['output_timestamp'] and item.input_timestamp is not None:
            name = ""
            if item.input_name is not None and item.input_name != "":
                name = item.input_name + " "
            ts = datetime.fromtimestamp(item.input_timestamp)
            hour = ts.strftime("%H:%M:%S")
            self.append("{}{}: > {}\n".format(name, hour, item.input))
        else:
            self.append("> {}\n".format(item.input))

    def append_output(self, item):
        """
        Appends output to output

        :param item: context item
        """
        if item.output is None or item.output == "":
            return
        if self.window.config.data['output_timestamp'] and item.output_timestamp is not None:
            name = ""
            if item.output_name is not None and item.output_name != "":
                name = item.output_name + " "
            ts = datetime.fromtimestamp(item.output_timestamp)
            hour = ts.strftime("%H:%M:%S")
            self.append("{}{}: {}".format(name, hour, item.output) + "\n")
        else:
            self.append(item.output + "\n")

    def append_chunk(self, item, text_chunk, begin=False):
        """
        Appends output to output

        :param item: context item
        :param text_chunk: text chunk
        :param begin: if it is the beginning of the text

        """
        if text_chunk is None or text_chunk == "":
            return
        if begin and self.window.config.data['output_timestamp'] and item.output_timestamp is not None:
            name = ""
            if item.output_name is not None and item.output_name != "":
                name = item.output_name + " "
            ts = datetime.fromtimestamp(item.output_timestamp)
            hour = ts.strftime("%H:%M:%S")
            self.append("{}{}: ".format(name, hour), "")

        self.append(text_chunk, "")

    def append(self, text, end="\n"):
        """
        Appends text to output

        :param text: text to append
        :param end: end of the line
        """
        cur = self.window.data['output'].textCursor()  # Move cursor to end of text
        cur.movePosition(QTextCursor.End)
        s = str(text) + end
        while s:
            head, sep, s = s.partition("\n")  # Split line at LF
            cur.insertText(head)  # Insert text at cursor
            if sep:  # New line if LF
                cur.insertBlock()
        self.window.data['output'].setTextCursor(cur)  # Update visible cursor

    def toggle_timestamp(self, value):
        """
        Toggles timestamp

        :param value: value of the checkbox
        """
        self.window.config.data['output_timestamp'] = value
        self.window.config.save()
        self.window.controller.context.refresh()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.06 12:00:00                  #
# ================================================== #
import os

from langchain.llms import HuggingFaceHub


class HuggingFaceLLM:
    def __init__(self):
        self.id = "huggingface"

    def completion(self, config, options, stream=False):
        args = {}
        if 'args' in options:
            args = options['args']
        os.environ['HUGGINGFACEHUB_API_TOKEN'] = options['api_key']
        llm = HuggingFaceHub(**args)
        return llm

    def chat(self, config, options, stream=False):
        return None


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.05 22:00:00                  #
# ================================================== #
import os
import threading

from PySide6.QtCore import QObject, Signal, Qt
from PySide6.QtWidgets import QCheckBox
from pydub import AudioSegment
from pydub.playback import play
from openai import OpenAI

from ..base_plugin import BasePlugin
from ...utils import trans


class Plugin(BasePlugin):
    def __init__(self):
        super(Plugin, self).__init__()
        self.id = "audio_openai_tts"
        self.name = "Audio Output (OpenAI TTS)"
        self.description = "Enables audio/voice output (speech synthesis) using OpenAI TTS (Text-To-Speech) API"
        self.allowed_voices = ['alloy', 'echo', 'fable', 'onyx', 'nova', 'shimmer']
        self.allowed_models = ['tts-1', 'tts-1-hd']
        self.options = {}
        self.options["model"] = {
            "type": "text",
            "slider": False,
            "label": "Model",
            "description": "Specify model, available models: tts-1, tts-1-hd",
            "tooltip": "Model",
            "value": 'tts-1',
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.options["voice"] = {
            "type": "text",
            "slider": False,
            "label": "Voice",
            "description": "Specify voice, available voices: alloy, echo, fable, onyx, nova, shimmer",
            "tooltip": "Voice",
            "value": 'alloy',
            "min": None,
            "max": None,
            "multiplier": None,
            "step": None,
        }
        self.input_text = None
        self.window = None
        self.order = 1

    def setup(self):
        """
        Returns available config options

        :return: config options
        """
        return self.options

    def setup_ui(self):
        """
        Setup UI

        :param ui: UI
        """
        pass

    def attach(self, window):
        """
        Attaches window

        :param window: Window
        """
        self.window = window

    def on_user_send(self, text):
        """Event: On user send text"""
        return text

    def on_ctx_begin(self, ctx):
        """Event: On new context begin"""
        return ctx

    def on_ctx_end(self, ctx):
        """Event: On context end"""
        return ctx

    def on_system_prompt(self, prompt):
        """Event: On prepare system prompt"""
        return prompt

    def on_ai_name(self, name):
        """Event: On set AI name"""
        return name

    def on_user_name(self, name):
        """Event: On set user name"""
        return name

    def on_enable(self):
        """Event: On plugin enable"""
        pass

    def on_disable(self):
        """Event: On plugin disable"""
        pass

    def on_input_before(self, text):
        """
        Event: Before input

        :param text: Text
        :return: text
        """
        self.input_text = text
        return text

    def on_ctx_before(self, ctx):
        """
        Event: Before ctx

        :param ctx: Text
        :return: ctx
        """
        return ctx

    def on_ctx_after(self, ctx):
        """
        Event: After ctx

        :param ctx: ctx
        :return: ctx
        """
        text = ctx.output
        try:
            if text is not None and len(text) > 0:
                client = OpenAI(
                    api_key=self.window.config.data["api_key"],
                    organization=self.window.config.data["organization_key"],
                )
                voice = self.options['voice']['value']
                path = os.path.join(self.window.config.path, 'output.mp3')
                model = self.options['model']['value']
                if model not in self.allowed_models:
                    model = 'tts-1'
                if voice not in self.allowed_voices:
                    voice = 'alloy'
                tts = TTS(client, model, path, voice, text)
                t = threading.Thread(target=tts.run)
                t.start()
        except Exception as e:
            print(e)

        return ctx

    def destroy(self):
        """
        Destroy thread
        """
        pass


class TTS(QObject):
    def __init__(self, client, model, path, voice, text):
        """
        Text to speech

        :param client: OpenAI client
        :param model: Model name
        :param voice: Voice name
        :param text: Text to speech
        """
        super().__init__()
        self.client = client
        self.model = model
        self.path = path
        self.text = text
        self.voice = voice

    def run(self):
        """Run TTS thread"""
        try:
            response = self.client.audio.speech.create(
                model="tts-1",
                voice=self.voice,
                input=self.text
            )
            response.stream_to_file(self.path)
            audio = AudioSegment.from_mp3(self.path)
            play(audio)
        except Exception as e:
            print(e)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.05 22:00:00                  #
# ================================================== #

from PySide6.QtGui import QAction

from ..utils import trans


class Lang:
    def __init__(self, window=None):
        """
        Language change controller

        :param window: main UI window object
        """
        self.window = window

    def setup(self):
        """Setups language handler"""
        # get files from lang folder
        langs = self.window.config.get_available_langs()
        for lang in langs:
            self.window.menu['lang'][lang] = QAction(lang.upper(), self.window, checkable=True)
            self.window.menu['lang'][lang].triggered.connect(
                lambda checked=None, lang=lang: self.window.controller.lang.toggle(lang))
            self.window.menu['menu.lang'].addAction(self.window.menu['lang'][lang])

        self.update()

    def update(self):
        """Updates language menu"""
        for lang in self.window.menu['lang']:
            self.window.menu['lang'][lang].setChecked(False)

        if self.window.config.data['lang'] in self.window.menu['lang']:
            self.window.menu['lang'][self.window.config.data['lang']].setChecked(True)

    def toggle(self, id):
        """
        Toggles language

        :param id: language to toggle
        """
        self.window.config.data['lang'] = id
        self.window.config.save()
        trans('', True)

        self.update()

        # output
        self.window.data['chat.label'].setText(trans("chatbox.label"))
        self.window.data['output.timestamp'].setText(trans('output.timestamp'))

        # context
        self.window.data['contexts.label'].setText(trans("ctx.contexts.label"))
        self.window.data['contexts.new'].setText(trans('context.new'))

        # toolbox
        self.window.data['prompt.mode.label'].setText(trans("toolbox.mode.label"))
        self.window.data['prompt.model.label'].setText(trans("toolbox.model.label"))
        self.window.data['preset.presets.label'].setText(trans("toolbox.presets.label"))
        self.window.data['preset.presets.new'].setText(trans('preset.new'))
        self.window.data['preset.clear'].setText(trans('preset.clear'))
        self.window.data['preset.use'].setText(trans('preset.use'))
        self.window.data['toolbox.preset.ai_name.label'].setText(trans('toolbox.name.ai'))
        self.window.data['toolbox.preset.user_name.label'].setText(trans('toolbox.name.user'))
        self.window.data['toolbox.prompt.label'].setText(trans("toolbox.prompt"))
        self.window.data['temperature.label'].setText(trans("toolbox.temperature.label"))

        # input
        self.window.data['input.label'].setText(trans("input.label"))
        self.window.data['input.send_enter'].setText(trans("input.radio.enter"))
        self.window.data['input.send_shift_enter'].setText(trans("input.radio.enter_shift"))
        self.window.data['input.send_clear'].setText(trans('input.send_clear'))
        self.window.data['input.send_btn'].setText(trans("input.btn.send"))

        # settings dialog
        self.window.data['settings.temperature.label'].setText(trans("settings.temperature"))
        self.window.data['settings.top_p.label'].setText(trans("settings.top_p"))
        self.window.data['settings.frequency_penalty.label'].setText(trans("settings.frequency_penalty"))
        self.window.data['settings.presence_penalty.label'].setText(trans("settings.presence_penalty"))
        self.window.config_option['use_context'].box.setText(trans("settings.use_context"))
        self.window.config_option['store_history'].box.setText(trans("settings.store_history"))
        self.window.config_option['store_history_time'].box.setText(trans("settings.store_history_time"))
        self.window.data['settings.context_threshold.label'].setText(trans("settings.context_threshold"))
        self.window.data['settings.max_output_tokens.label'].setText(trans("settings.max_output_tokens"))
        self.window.data['settings.max_total_tokens.label'].setText(trans("settings.max_total_tokens"))
        self.window.data['settings.img_resolution.label'].setText(trans("settings.img_resolution"))
        self.window.data['settings.api_key.label'].setText(trans("settings.api_key"))
        self.window.data['settings.organization_key.label'].setText(trans("settings.organization_key"))
        self.window.dialog['config.settings'].setWindowTitle(trans('dialog.settings'))

        # buttons
        self.window.data['settings.btn.defaults'].setText(trans("dialog.settings.btn.defaults"))
        self.window.data['settings.btn.save'].setText(trans("dialog.settings.btn.save"))

        # preset dialog
        self.window.data['preset.filename.label'].setText(trans("preset.filename"))
        self.window.data['preset.name.label'].setText(trans("preset.name"))
        self.window.data['preset.ai_name.label'].setText(trans("preset.ai_name"))
        self.window.data['preset.user_name.label'].setText(trans("preset.user_name"))
        self.window.config_option['preset.chat'].box.setText(trans("preset.chat"))
        self.window.config_option['preset.completion'].box.setText(trans("preset.completion"))
        self.window.config_option['preset.img'].box.setText(trans("preset.img"))
        self.window.data['preset.temperature.label'].setText(trans("preset.temperature"))
        self.window.data['preset.prompt.label'].setText(trans("preset.prompt"))
        self.window.dialog['editor.preset.presets'].setWindowTitle(trans('dialog.preset'))

        # buttons
        self.window.data['preset.btn.current'].setText(trans("dialog.preset.btn.current"))
        self.window.data['preset.btn.save'].setText(trans("dialog.preset.btn.save"))

        # editor dialog
        self.window.data['dialog.editor.label'].setText(trans('dialog.editor.label'))
        self.window.data['editor.btn.default'].setText(trans("dialog.editor.btn.defaults"))
        self.window.data['editor.btn.save'].setText(trans("dialog.editor.btn.save"))
        self.window.dialog['config.editor'].setWindowTitle(trans('dialog.editor.title'))

        # rename dialog
        self.window.dialog['rename'].setWindowTitle(trans("dialog.rename.title"))
        self.window.data['dialog.rename.label'].setText(trans("dialog.rename.title"))
        self.window.data['dialog.rename.btn.update'].setText(trans("dialog.rename.update"))
        self.window.data['dialog.rename.btn.dismiss'].setText(trans("dialog.rename.dismiss"))

        # changelog dialog
        self.window.data['dialog.changelog.label'].setText(trans("dialog.changelog.title"))
        self.window.dialog['info.changelog'].setWindowTitle(trans("dialog.changelog.title"))

        # about dialog
        string = "PY-GPT\n" \
                 "-------------\n" \
                 "{}: {}\n" \
                 "{}: {}\n" \
                 "{}: {}\n" \
                 "{}: {}\n\n" \
                 "(c) 2023 {}\n" \
                 "{}\n".format(trans("dialog.about.version"),
                               self.window.version,
                               trans("dialog.about.build"),
                               self.window.build,

                               trans("dialog.about.website"),
                               self.window.website,
                               trans("dialog.about.github"),
                               self.window.github,
                               self.window.author,
                               self.window.email)
        self.window.data['dialog.about.content'].setText(string)
        self.window.dialog['info.about'].setWindowTitle(trans("dialog.about.title"))

        # menu
        self.window.menu['menu.app'].setTitle(trans("menu.file"))
        self.window.menu['app.exit'].setText(trans("menu.file.exit"))
        self.window.menu['app.clear_history'].setText(trans("menu.file_clear_history"))

        self.window.menu['menu.plugins'].setTitle(trans("menu.plugins"))

        self.window.menu['menu.audio'].setTitle(trans("menu.audio"))
        self.window.menu['audio.output'].setText(trans("menu.audio.output"))

        self.window.menu['menu.config'].setTitle(trans("menu.config"))
        self.window.menu['config.settings'].setText(trans("menu.config.settings"))
        self.window.menu['config.edit.config'].setText(trans("menu.config.edit.config"))
        self.window.menu['config.edit.models'].setText(trans("menu.config.edit.models"))
        self.window.menu['config.open_dir'].setText(trans("menu.config.open_dir"))
        self.window.menu['config.save'].setText(trans("menu.config.save"))

        self.window.menu['menu.lang'].setTitle(trans("menu.lang"))
        self.window.menu['menu.theme'].setTitle(trans("menu.theme"))

        if 'menu.debug' in self.window.menu:
            self.window.menu['menu.debug'].setTitle(trans("menu.debug"))
            self.window.menu['debug.config'].setText(trans("menu.debug.config"))
            self.window.menu['debug.context'].setText(trans("menu.debug.context"))
            self.window.menu['debug.presets'].setText(trans("menu.debug.presets"))
            self.window.menu['debug.models'].setText(trans("menu.debug.models"))

        self.window.menu['menu.about'].setTitle(trans("menu.info"))
        self.window.menu['info.about'].setText(trans("menu.info.about"))
        self.window.menu['info.changelog'].setText(trans("menu.info.changelog"))
        self.window.menu['info.website'].setText(trans("menu.info.website"))
        self.window.menu['info.github'].setText(trans("menu.info.github"))

        # start
        self.window.data['start.title'].setText(trans('dialog.start.title.text'))
        self.window.data['start.settings'].setText(trans('dialog.start.settings.text'))
        self.window.data['start.btn'].setText(trans('dialog.start.btn'))

        # input tabs
        self.window.tabs['input'].setTabText(0, trans('input.tab'))
        self.window.tabs['input'].setTabText(1, trans('attachments.tab'))
        self.window.data['attachments.btn.add'].setText(trans('attachments.btn.add'))
        self.window.data['attachments.btn.clear'].setText(trans('attachments.btn.clear'))

        # plugins info
        self.window.controller.plugins.update_info()

        for theme in self.window.menu['theme']:
            name = self.window.controller.theme.trans_theme(theme)
            self.window.menu['theme'][theme].setText(name)

        self.window.controller.model.update()
        self.window.set_status('')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.08 22:00:00                  #
# ================================================== #

from PySide6 import QtCore
from PySide6.QtWidgets import QLineEdit, QTextEdit, QApplication, QTextBrowser


class NameInput(QLineEdit):
    def __init__(self, window=None, id=None):
        """
        AI or user name input

        :param window: main window
        :param id: input id
        """
        super(NameInput, self).__init__(window)
        self.window = window
        self.id = id
        self.setStyleSheet(self.window.controller.theme.get_style('line_edit'))

    def keyPressEvent(self, event):
        """
        Key press event

        :param event: key event
        """
        super(NameInput, self).keyPressEvent(event)
        self.window.controller.ui.update()
        self.window.controller.presets.update_field(self.id, self.text(), self.window.config.data['preset'], True)


class ChatInput(QTextEdit):
    def __init__(self, window=None):
        """
        Chat input

        :param window: main window
        """
        super(ChatInput, self).__init__(window)
        self.window = window
        self.setFocus()

    def keyPressEvent(self, event):
        """
        Key press event

        :param event: key event
        """
        super(ChatInput, self).keyPressEvent(event)
        self.window.controller.ui.update()
        if event.key() == QtCore.Qt.Key_Return or event.key() == QtCore.Qt.Key_Enter:
            if self.window.config.data['send_mode'] > 0:
                if self.window.config.data['send_mode'] == 2:
                    modifiers = QApplication.keyboardModifiers()
                    if modifiers == QtCore.Qt.ShiftModifier:
                        self.window.controller.input.user_send()
                else:
                    self.window.controller.input.user_send()
                self.setFocus()


class ChatOutput(QTextBrowser):
    def __init__(self, window=None):
        """
        Chat output

        :param window: main window
        """
        super(ChatOutput, self).__init__(window)
        self.window = window
        self.setReadOnly(True)
        self.setStyleSheet(self.window.controller.theme.get_style('chat_output'))


class NotepadOutput(QTextEdit):
    def __init__(self, window=None):
        """
        Notepad

        :param window: main window
        """
        super(NotepadOutput, self).__init__(window)
        self.window = window
        self.setStyleSheet(self.window.controller.theme.get_style('chat_output'))

    def keyPressEvent(self, event):
        """
        Key press event

        :param event: key event
        """
        super(NotepadOutput, self).keyPressEvent(event)
        self.window.controller.notepad.save()


class RenameInput(QLineEdit):
    def __init__(self, window=None, id=None):
        """
        Rename dialog

        :param window: main window
        :param id: info window id
        """
        super(RenameInput, self).__init__(window)

        self.window = window
        self.id = id
        self.setStyleSheet(self.window.controller.theme.get_style('line_edit'))

    def keyPressEvent(self, event):
        """
        Key press event

        :param event: key event
        """
        super(RenameInput, self).keyPressEvent(event)
        if event.key() == QtCore.Qt.Key_Return or event.key() == QtCore.Qt.Key_Enter:
            self.window.controller.confirm.accept_rename(self.window.dialog['rename'].id,
                                                         self.window.dialog['rename'].current,
                                                         self.text())
